@@ -27,11 +27,13 @@
 struct emitter_xtra {
     SV* port;
     char* tag;
+	char dump_code;
 };
 
 struct parser_xtra {
    AV *objects;
    int utf8;
+   char load_code;
 };
 
 SV* perl_syck_lookup_sym( SyckParser *p, SYMID v) {
@@ -62,7 +64,9 @@ void perl_syck_mark_emitter(SyckEmitter *e, SV *sv) {
             len = av_len((AV*)sv) + 1;
             for (i = 0; i < len; i++) {
                 SV** sav = av_fetch((AV*)sv, i, 0);
-                perl_syck_mark_emitter( e, *sav );
+                if (sav != NULL) {
+                    perl_syck_mark_emitter( e, *sav );
+                }
             }
             break;
         }
